from .zxcvbn_rs_py import (
    CrackTimesDisplay,
    CrackTimesSeconds,
    Entropy,
    Feedback,
    Suggestion,
    Warning,
    __version__,
    zxcvbn,
)

__all__ = [
    "CrackTimesDisplay",
    "CrackTimesSeconds",
    "Entropy",
    "Feedback",
    "Suggestion",
    "Warning",
    "__version__",
    "zxcvbn",
]



import pytest

from zxcvbn_rs_py import zxcvbn


@pytest.mark.parametrize(
    "password,score",
    [
        ("correcthorsebatterystaple", 4),
    ],
)
def test_zxcvbn_rs_py(password: str, score: int) -> None:
    r = zxcvbn("correcthorsebatterystaple")
    assert r.score == score



from enum import Enum, auto

__version__: str

class Warning(Enum):
    StraightRowsOfKeysAreEasyToGuess = auto()
    ShortKeyboardPatternsAreEasyToGuess = auto()
    RepeatsLikeAaaAreEasyToGuess = auto()
    RepeatsLikeAbcAbcAreOnlySlightlyHarderToGuess = auto()
    ThisIsATop10Password = auto()
    ThisIsATop100Password = auto()
    ThisIsACommonPassword = auto()
    ThisIsSimilarToACommonlyUsedPassword = auto()
    SequencesLikeAbcAreEasyToGuess = auto()
    RecentYearsAreEasyToGuess = auto()
    AWordByItselfIsEasyToGuess = auto()
    DatesAreOftenEasyToGuess = auto()
    NamesAndSurnamesByThemselvesAreEasyToGuess = auto()
    CommonNamesAndSurnamesAreEasyToGuess = auto()

class Suggestion(Enum):
    UseAFewWordsAvoidCommonPhrases = auto()
    NoNeedForSymbolsDigitsOrUppercaseLetters = auto()
    AddAnotherWordOrTwo = auto()
    CapitalizationDoesntHelpVeryMuch = auto()
    AllUppercaseIsAlmostAsEasyToGuessAsAllLowercase = auto()
    ReversedWordsArentMuchHarderToGuess = auto()
    PredictableSubstitutionsDontHelpVeryMuch = auto()
    UseALongerKeyboardPatternWithMoreTurns = auto()
    AvoidRepeatedWordsAndCharacters = auto()
    AvoidSequences = auto()
    AvoidRecentYears = auto()
    AvoidYearsThatAreAssociatedWithYou = auto()
    AvoidDatesAndYearsThatAreAssociatedWithYou = auto()

class Feedback:
    warning: Warning | None
    suggestions: list[Suggestion]

class CrackTimesSeconds:
    offline_fast_hashing_1e10_per_second: float
    offline_slow_hashing_1e4_per_second: float
    online_no_throttling_10_per_second: float
    online_throttling_100_per_hour: float

class CrackTimesDisplay:
    offline_fast_hashing_1e10_per_second: str
    offline_slow_hashing_1e4_per_second: str
    online_no_throttling_10_per_second: str
    online_throttling_100_per_hour: str

class Entropy:
    """Result of a password strength check."""

    guesses: int
    """Estimated guesses needed to crack the password"""

    guesses_log10: float
    """Order of magnitude of `guesses`."""

    crack_times_seconds: CrackTimesSeconds
    """List of back-of-the-envelope crack time estimations based on a few scenarios."""

    crack_times_display: CrackTimesDisplay
    """Same as crack_times_seconds, with friendlier display string values."""

    score: int
    """
    Overall strength score from 0-4.
    Any score less than 3 should be considered too weak.
    """

    feedback: Feedback | None
    """Verbal feedback to help choose better passwords. Set when `score` <= 2."""

    calc_time: int
    """How long it took to calculate the answer."""

def zxcvbn(password: str, user_inputs: list[str] | None = None) -> Entropy:
    """
    Measure the strength of a password.

    Args:
        password: The password to check.
        user_inputs: Optional list of strings input by the user
        in other fields to match against the password.

    Returns:
        An `Entropy` object.
    """
    ...



import csv
import timeit

LIBRARIES = [
    "zxcvbn",
    "zxcvbn_rs_py",
]

PASSWORDS = {
    "Length 4": "igrI",
    "Length 8": "UzA170rl",
    "Length 16": "pvmXR8ECp10UIRCs",
    "Length 32": "MtvEJJUM2K0mKAyWKy0emf6Bt9xyxrru",
    "Repetition": "a" * 16,
}

NB_LOOP = 500
REPEAT = 5

with open("benchmark/benchmark.dat", "w") as file:
    writer = csv.writer(file, delimiter="\t", quoting=csv.QUOTE_NONNUMERIC)
    writer.writerow(("Case", *LIBRARIES))
    for key, value in PASSWORDS.items():
        row = [key]
        print(f"Run benchmark for {key}...")
        for library in LIBRARIES:
            result = min(
                timeit.repeat(
                    f"zxcvbn('{value}')",
                    setup=f"from {library} import zxcvbn",
                    number=NB_LOOP,
                    repeat=REPEAT,
                )
            )
            row.append(result / NB_LOOP * 1000)
        writer.writerow(row)
